#!/usr/bin/env python

class IoException(Exception):
    """Base class for Io related exceptions."""

class IoTimeoutException(IoException):
    """Io timeouts give an exception"""

timeoutError = IoTimeoutException("timeout")

class IoConnectionAbortedException(IoException):
    """Io give an exception"""
connectionAbortedError = IoConnectionAbortedException("Connection aborted")

class IoNotAvailable(IoException):
    """The port or device is not available"""
    def __init__(self, portName):
        self._message = "Port or device not available (%s)" % portName 

    def __str__(self):
        return self._message
    
class IoBase(object):
    """Io base class."""

    def get_initialisied(self): return self._initialised
    initialised = property(get_initialisied)
    def get_connected(self): return self._connected
    connected = property(get_connected)

    def __init__(self, **kwargs):
        self._initialised = False
        self._connected = False

    def open(self): raise NotImplementedError
    def close(self): raise NotImplementedError
    def read(self, size=0): raise NotImplementedError
    def write(self, s): raise NotImplementedError

    def flushInput(self): raise NotImplementedError
    def inWaiting(self): raise NotImplementedError
    def setTimeout(self, timeout): raise NotImplementedError
