#!/usr/bin/env python

"""**********************************************************************
  DLMS/COSEM components

  i-cube, 1376 Goumoens-la-Ville, Switzerland
          infos@icube.ch

 (C)Copyright 2004-2012
**********************************************************************

"""

import binascii
import os
import serial
import string
import sys
import time
import types
import xml.etree.cElementTree as ET

from icube.memorystream import PduOutputStream, PduInputStream

from icube.utils import Utils
import icube.ezhdlc
import icube.xmlpdu
import icube.wrapper
from icube.xmlpdu.auto.XMLIDs import XmlIDs
from icube.xmlpdu.auto._COSEMpdu import _COSEMpdu
from icube.xmlpdu.infra.xmlstringbuffer import XmlStringBuffer

from icube.ezhdlc import HDLCParameters, HDLCProtocolState
from icube.ezhdlc.hdlcprotocol import HDLCProtocol
from msilib.schema import Property


class HdlcDemo(object):
    
    def __init__(self, iec=False):
        self._iec = iec
        self._iec_parameters = None
        self._initialize_parameters(iec=iec)
        
    def OnSendStream(self, stream):
        if isinstance(stream, types.ListType):
            print "CB_S:", Utils.bytes_to_hex_string(stream)
        else:
            print "CB_S:", stream
            
    def OnReceivedStream(self, stream):
        if isinstance(stream, types.ListType):
            print "CB_R:", Utils.bytes_to_hex_string(stream)
        else:
            print "CB_R:", stream

    def OnStateChange(self, state):
        print "CB_s:", HDLCProtocolState.str(state)

    def _initialize_parameters(self, iec=True):
        self._port_name = 'COM1' # 'COM12'
        self._hdlcParamaters = HDLCParameters()
        self._hdlcParamaters.SentStreamCallback = self.OnSendStream
        self._hdlcParamaters.ReceivedStreamCallBack = self.OnReceivedStream
        self._hdlcParamaters.StateChangeCallBack = self.OnStateChange
    
        self._hdlcParamaters.ResponseTimeout = 2000
        self._hdlcParamaters.InterFrameTimeOut = 50
        self._hdlcParamaters.InactivityTimeout = 1000
        self._hdlcParamaters.DiscToNDMTimeout = 1000
    
        self._hdlcParamaters.ClientMACAddress = 0x10
    
        self._hdlcParamaters.ServerAddressSize = 1
        self._hdlcParamaters.ServerUpperMACAddressValue = 1
        self._hdlcParamaters.ServerLowerMACAddressValue = 0
        
        if iec:
            self._iec_parameters = self.IECParameters()
            self._iec_parameters.opening_baud = 300
            self._iec_parameters.max_baud = 115200
            self._iec_parameters.swap_baud_delay = 0.300 # in seconds
            self._iec_parameters.mode_E_device_address = ""
            self._iec_parameters.is_echoing = False
        
    def process_xml_file(self, file_):
        if True:
            print 6*'-----     '
            print "process_file: %s" % file_
            with open(file_) as xml:
                for line in xml.readlines():
                    print line.rstrip()
                    
            _tree = ET.parse(file_)
            _root = _tree.getroot()
            _pdu = _COSEMpdu(XmlIDs.XMLIgnore, element=_root)
            _pduStream = PduOutputStream()
            _pdu.to_pdu(_pduStream)
            
            _inPdu = self._hdlc.send_receive(_pduStream._buffer)
            
            _inPdu = PduInputStream(_inPdu._buffer)
            _pduFromBytes = _COSEMpdu(XmlIDs.XMLIgnore, pdu=_inPdu)
            _xml = XmlStringBuffer()
            _pduFromBytes.to_xml(_xml)
            _xmlStr = _xml.buffer
            print _xmlStr
            
#        except:
#            cla, exc, trbk =  sys.exc_info()
#            print exc

    def doIt(self, files):
        print "HdlcDemo"
        print icube.ezhdlc.get_version_str()
        print icube.xmlpdu.get_version_str()
        print icube.wrapper.get_version_str()
        print

        if not self._iec:
            IOStream = serial.Serial(self._port_name, baudrate=9600, bytesize=8, parity='N', stopbits=1, timeout=5)
        else:
            iec_opening = self.IECOpening(self._port_name, self._iec_parameters)
            IOStream = iec_opening.IOStream
        self._hdlc = HDLCProtocol(self._hdlcParamaters, IOStream=IOStream)
        self._hdlc.connect()
        
        try:
            for _file in files:
                self.process_xml_file(_file)
        finally:
            self._hdlc.disconnect()
            IOStream.close()

    class IECParameters(object):
        @property 
        def opening_baud(self): return self._opening_baud
        @opening_baud.setter
        def opening_baud(self, value): self._opening_baud = value
        
        @property
        def max_baud(self): return self._max_baud
        @max_baud.setter
        def max_baud(self, value): self._max_baud = value
        
        @property
        def swap_baud_delay(self): return self._swap_baud_delay
        @swap_baud_delay.setter
        def swap_baud_delay(self, value): self._swap_baud_delay = value
        
        @property
        def mode_E_device_address(self): return self._mode_E_device_address
        @mode_E_device_address.setter
        def mode_E_device_address(self, value): self._mode_E_device_address = value
        
        @property
        def is_echoing(self): return self._is_echoing
        @is_echoing.setter
        def is_echoing(self, value): self._is_echoing = value
                
    class IECOpening(object):
        
        class IECException(Exception):
            pass
        
        _HW_MISSING_ = False
        
        _CR_ = 0x0D
        _LF_ = '\n' # 0x0A
        _ACK_ = 0x06
        
        _NON_PRINTABLE_ASCII_ = [
            "NUL",  "SOH",  "STX",  "ETX",  "EOT",  "ENQ",  "ACK",  "BEL",
            "BS",   "HT",   "LF",   "VT",   "FF",   "CR",   "SO",   "SI",
            "DLE",  "DC1",  "DC2",  "DC3",  "DC4",  "NAK",  "SYN",  "ETB",
            "CAN",  "EM",   "SUB",  "ESC",  "FS",   "GS",   "RS",   "US",
        ]
        _CHAR_TO_BAUD_ = { 
            '0': 300, '1': 600, '2': 1200,
            '3': 2400, '4': 4800, '5': 9600,
            '6': 19200, '7': 38400, '8': 57600, '9': 115200
        }
        
        @property
        def IOStream(self): return self._IOStream
        
        def __init__(self, port_name, iec_parameters):
            self._iec_parameters = iec_parameters
            self._IOStream = serial.Serial(port_name, 
                                     baudrate=iec_parameters.opening_baud, 
                                     bytesize=7, 
                                     parity='E', 
                                     stopbits=1, 
                                     timeout=3)
            self._open()
        
        def _open(self):
            # send identification string
            s = "/?" + self._iec_parameters.mode_E_device_address + "!\r\n"
            
            print "--> {}".format(self._get_readable_ASCII_from_string(s))
            
            self._write(bytearray(s))
            
            # get and check response
            response, timeout = self._get_response(self._LF_)
            
            if timeout:
                raise self.IECException('No response to mode E opening')
            
            if len(response) < 6:
                raise self.IECException('Mode E illegal response: lg')
            if response[0] != ord('/'):
                raise self.IECException('Mode E illegal response: slash')
            
            baud_char = chr(response[4])
            if baud_char not in '0123456789':
                raise self.IECException('Mode E illegal response: baud')

#            print 'baud: %s: %d' % ( baud_char, self._CHAR_TO_BAUD_[baud_char] )
#            _ACK_STRING_ = [ _ACK_, ord('2'), 0, ord('2'), _CR_, _LF_ ]
            ackowledge = '\x062{}2\r\n'.format(baud_char)
            
            print "--> {}".format(self._get_readable_ASCII_from_string(ackowledge))
            if self._iec_parameters.swap_baud_delay != 0:
                time.sleep(self._iec_parameters.swap_baud_delay)
            self._write(ackowledge)
            self._IOStream.flushOutput()
            
            time.sleep(2.0)
            self._IOStream.flushInput()

            self._IOStream.baudrate = self._CHAR_TO_BAUD_[baud_char]
            self._IOStream.bytesize = 8
            self._IOStream.parity = 'N'
            
            print '--- baud changeover --- {}\n'.format(self._CHAR_TO_BAUD_[baud_char])
            
            
        def _write(self, bytes_):
            """Write the bytes array
                @note: implement 'is_echoing' here if necessary
            """
            if not self._HW_MISSING_:
                self._IOStream.write(bytes_)
            else:
                self._IOStream.write(bytes_)

        def _get_response(self, last_char):
            response = bytearray()
            timeout = False
            done = False
            if self._HW_MISSING_:
                response = bytearray('/LGZ5\\2ZMD3100007.B22\r\n')
            else:
                while not done:
                    bb = self._IOStream.read(1)
                    timeout = (len(bb) == 0)
                    if timeout: 
                        break
                    b = bb[0]
                    response += bb
                    done = (last_char == b)
                    
            print "<-- {}".format(self._get_readable_ASCII_from_stream(response))
            return response, timeout
        
        def _get_readable_ASCII_from_string(self, s):
#            bytes_ = [ ord(c) for c in s ]
            bytes_ = bytearray(s)
            return self._get_readable_ASCII_from_stream(bytes_)
        
        def _get_readable_ASCII_from_stream(self, stream):
            s = ''
            for b in stream:
                if b < len(self._NON_PRINTABLE_ASCII_):
                    s += '<' + self._NON_PRINTABLE_ASCII_[b] + '>'
                else:
                    s += chr(b)
            return s
        
if __name__ == '__main__':
    print sys.argv

    def usage():
        """
    usage: 
        python hdlcdemo.py file(s)
            connect to server, process file(s) 
                xml->pdu
                pdu->server->inpdu
                inpdu->xml 
     """
        print usage.__doc__

#    print os.getcwd(), len(sys.argv), sys.argv
    iec = False
    if len(sys.argv) >= 1:
        HdlcDemo(iec=iec).doIt(sys.argv[1:])
    else:
        usage()

